import 'package:dio/dio.dart';
import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';

import '../fast_mvvm.dart';
import 'base.dart';

/// 接口数据 响应
class DataResponse<T> {
  T? entity;

  /// 最终结果
  bool result;

  /// dio 的默认
  Response? response;

  /// 当前页面总页码  配合 [BaseListViewModel] 默认一页
  int? totalPageNum;

  /// 扩展数据
  Map<String, dynamic>? extend;

  /// 语法糖
  get data => response?.data;

  /// 语法糖 默认的[Response]的statusCode 可以在 [InterceptorsWrapper] 自定义处理
  int? get code => response?.statusCode;

  DataResponse({
    required this.entity,
    this.result = false,
    this.response,
    this.totalPageNum = 1,
  });

  /// 拷贝
  DataResponse.copy({
    required DataResponse dataResponse,
    required this.entity,
    required this.result,
    this.response,
    this.totalPageNum,
    this.extend,
  }) {
    response ??= dataResponse.response;
    totalPageNum ??= dataResponse.totalPageNum;
    extend ??= dataResponse.extend;
  }
}

/// view层 配置用类  配置全局默认状态页
class ViewConfig<VM extends BaseViewModel> {
  ViewConfig(
    this.vm, {
    this.child,
    this.color,
    this.load = true,
    this.checkEmpty = true,
    this.state,
    this.value = false,
    this.busy,
    this.empty,
    this.error,
    this.unAuthorized,
    this.isStatusPage,
    root = true,
  }) {
    setViewState();
    vm.rootRefresh = root;
  }

  ViewConfig.value({
    this.child,
    this.color,
    this.load = false,
    this.checkEmpty = true,
    this.state,
    this.value = true,
    this.busy,
    this.empty,
    this.error,
    this.unAuthorized,
    this.isStatusPage,
    root = true,
  }) {
    setViewState();
    vm.rootRefresh = root;
  }

  ViewConfig.noRoot(
    this.vm, {
    this.child,
    this.color,
    this.load = true,
    this.checkEmpty = true,
    this.state,
    this.value = false,
    this.busy,
    this.empty,
    this.error,
    this.unAuthorized,
    this.isStatusPage,
    root = false,
  }) {
    setViewState();
    vm.rootRefresh = root;
  }

  ViewConfig.noLoad(
    this.vm, {
    this.child,
    this.color,
    this.load = false,
    this.checkEmpty = true,
    this.state,
    this.value = false,
    this.busy,
    this.empty,
    this.error,
    this.unAuthorized,
    this.isStatusPage,
    root = true,
  }) {
    setViewState();
    vm.rootRefresh = root;
  }

  /// VM
  late VM vm;

  Widget? child;

  /// 背景颜色
  Color? color;

  /// 加载
  bool load;

  /// [ChangeNotifierProvider.value] 或者[ChangeNotifierProvider]
  bool value;

  /// 是否验证空数据
  bool checkEmpty;

  /// 页面变化控制  可以被其他页面控制刷新
  int? state;

  ///是否自己处理状态页面
  bool? isStatusPage;

  static VSBuilder? gBusy;
  static VSBuilder? gEmpty;
  static VSBuilder? gError;
  static VSBuilder? gunAuthorized;
  static bool gIsStatusPage = false;

  /// 列表页  列表数据空
  static VSBuilder? gListDataEmpty;

  VSBuilder<VM>? busy;
  VSBuilder<VM>? empty;
  VSBuilder<VM>? error;
  VSBuilder<VM>? unAuthorized;

  void setViewState() {
    busy ??= gBusy;
    empty ??= gEmpty;
    error ??= gError;
    unAuthorized ??= gunAuthorized;
    isStatusPage ??= gIsStatusPage;
  }
}

/// 获取可用的监听 [ChangeNotifierProvider.value] 或者 [ChangeNotifierProvider]
ChangeNotifierProvider availableCNP<T extends BaseViewModel>(
    BuildContext context, ViewConfig<T> changeNotifier,
    {required Widget? child}) {
  if (changeNotifier.value) {
    changeNotifier.vm = Provider.of<T>(context);
    return ChangeNotifierProvider<T>.value(
        value: changeNotifier.vm, child: child);
  } else {
    return ChangeNotifierProvider<T>(
        create: (_) => changeNotifier.vm, child: child);
  }
}

class _ViewStateNotifier {
  ValueNotifier<bool> vn;
  bool notifier;

  _ViewStateNotifier(this.vn, {this.notifier = true});
}

/// 状态通知 跨页面通知数据需要变动
class ViewStateNotifier {
  bool changer;
  bool notifier;

  ViewStateNotifier(this.changer, this.notifier);
}

/// 全局状态变动存储
Map<int, _ViewStateNotifier> _changerState = {};

/// 获取状态配置
// ignore: library_private_types_in_public_api
_ViewStateNotifier changerStateGet(int state) {
  if (!_changerState.containsKey(state)) {
    _changerState[state] = _ViewStateNotifier(ValueNotifier(false));
  }
  return _changerState[state]!;
}

/// 更新页面状态
void changerStateUpdate(int state, {bool notifier = true}) {
  if (_changerState.containsKey(state)) {
    _changerState[state]!.vn.value = true;
    _changerState[state]!.notifier = notifier;
  }
}

/// 验证是否需要变化
ViewStateNotifier changerStateCheck(int state) {
  var result = _changerState[state]!.vn.value;
  _changerState[state]?.vn.value = false;
  return ViewStateNotifier(result, _changerState[state]!.notifier);
}

/// 全局Model管理
Map<String, BaseModel> _mList = {};

/// 添加Model
void addModel({required List<BaseModel> list}) {
  for (var element in list) {
    _addModel(element);
  }
}

_addModel(BaseModel model) => _mList[model.runtimeType.toString()] = model;

/// 全局Model
M getModelGlobal<M extends BaseModel>() => _mList[M.toString()] as M;

/// 得到通知者
T getVM<T extends ChangeNotifier>(BuildContext ctx, {bool listen = false}) =>
    Provider.of<T>(ctx, listen: listen);
